
from types import *
from MLab import *
from Numeric import *
from ve08ad import *

# Read ve08ad.f for description of variables.
# nvar need only be the starting indices for the variables, and
# the class supplies the ns+1 element.

class BoundedMinimization(DictionaryType):
   def __init__(self):
      self["fx"] = array((0,),Float64)
      self["ipdevc"] = 6
      self["ipfreq"] = -1
      self["ipwhat"] = 1
      self["epsil"] = 1e-8
      self["ebound"] = -1
      self["fknown"] = 0
      self["flowbd"] = 0
      self["relpr"] = -1
      self["difgrd"] = -1
      self["restrt"] = 0
      self["testgx"] = 0
      self["hesdif"] = 0
      self["stmax"] = 1e10
      self["ngr"] = asarray([3000,0],Int32)
      self["nit"] = asarray([1000,0],Int32)
      self["stepl"] = asarray([1e5,0],Float64)
      self["funcs"] = ()

   def __repr__(self):
      return ve08ad.__doc__

   def __doc__(self):
      return ve08ad.__doc__

   def __setitem__(self,item,value):
      self.__setattr__(item,value)
      if sum([k in dir(self) for k in ["ns","invar","nvar","n"]]) == 4 and "lwk" not in dir(self):
         self.setlwk()

   def check(self):
      return self.keys()

   def setlwk(self):
      self.nvar = concatenate([self.nvar,[len(self.invar)+1]]).astype(Int32)
      nv = self.nvar[self.ns]-1
      ne = max(diff(self.nvar))
      nq = max([2*self.n,nv])
      nwrk = self.ns+nv+4*self.n+nq+3*ne + (self.n*self.ns)
      self.lwk = asarray([nwrk,0],Int32)
      self.wk = zeros((self.lwk[0],),Float64)
      self.invar = concatenate([self.invar,zeros((self.n*self.ns-len(self.invar),),Int32)])

   def run(self):
      self.x = asarray(self.x).astype(Float64)
      if len(self.istate) != (self.n + self.ns):
         print "Error in self.istate: not long enough!";return 0,0
      self.iflag, self.info = ve08ad(self.n, self.ns, self.invar, self.nvar,
         self.elfnct, self.range, self.xlower, self.xupper, self.x,
         self.fx, self.epsil, self.ebound, self.ngr, self.nit,
         self.fknown, self.flowbd, self.relpr, self.difgrd, self.restrt,
         self.testgx, self.hesdif, self.stmax, self.stepl, self.istate,
         self.ipdevc, self.ipfreq, self.ipwhat, self.lwk, self.wk,
         elfnct_extra_args=(self.funcs,))
      return self.iflag, self.info

